@@ -3,7 +3,7 @@ from __future__ import unicode_literals
 import unittest
 import re
 import sys
-import pep8
+import pycodestyle
 
 from fuzzywuzzy import fuzz
 from fuzzywuzzy import process
@@ -498,10 +498,20 @@ class ProcessTest(unittest.TestCase):
         result = process.dedupe(contains_dupes)
         self.assertEqual(result, deduped_list)
 
+    def test_simplematch(self):
+        basic_string = 'a, b'
+        match_strings = ['a, b']
+
+        result = process.extractOne(basic_string, match_strings, scorer=fuzz.ratio)
+        part_result = process.extractOne(basic_string, match_strings, scorer=fuzz.partial_ratio)
+
+        self.assertEqual(result, ('a, b', 100))
+        self.assertEqual(part_result, ('a, b', 100))
+
 
 class TestCodeFormat(unittest.TestCase):
     def test_pep8_conformance(self):
-        pep8style = pep8.StyleGuide(quiet=True)
+        pep8style = pycodestyle.StyleGuide(quiet=False)
         pep8style.options.ignore = pep8style.options.ignore + tuple(['E501'])
         pep8style.input_dir('fuzzywuzzy')
         result = pep8style.check_files()
